/******************************************************************************
 *
 * Project:  OpenCPN
 * Purpose:  GE2KAP Plugin
 * Author:   Paul Higgins     
 *
 ***************************************************************************
 *   Copyright (C) 2010 by Paul Higginsh                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA.             *
 ***************************************************************************
 */


#include "wx/wxprec.h"

#ifndef  WX_PRECOMP
  #include "wx/wx.h"
#endif //precompiled headers


#include <wx/treectrl.h>
#include <wx/fileconf.h>

#include "ge2kap_pi.h"
#include "ge2kap.h"


// the class factories, used to create and destroy instances of the PlugIn

extern "C" DECL_EXP opencpn_plugin* create_pi(void *ppimgr)
{
    return new ge2kap_pi(ppimgr);
}

extern "C" DECL_EXP void destroy_pi(opencpn_plugin* p)
{
    delete p;
}

//---------------------------------------------------------------------------------------------------------
//
//    ge2kap PlugIn Implementation
//
//---------------------------------------------------------------------------------------------------------

#include "icons.h"



//---------------------------------------------------------------------------------------------------------
//
//          PlugIn initialization and de-init
//
//---------------------------------------------------------------------------------------------------------

ge2kap_pi::ge2kap_pi(void *ppimgr)
      :opencpn_plugin_17(ppimgr)
{
      // Create the PlugIn icons
      initialize_images();

}

ge2kap_pi::~ge2kap_pi(void)
{
      delete _img_ge2kap_pi; 
      delete _img_ge2kap;
}

int ge2kap_pi::Init(void)
{
      m_bshuttingDown = false;
//      printf("ge2kap_pi Init()\n");
      AddLocaleCatalog( _T("opencpn-ge2kap_pi") );

      // Set some default private member parameters
      m_ge2kap_dialog_x = 0;
      m_ge2kap_dialog_y = 0;
      m_ge2kap_dialog_sx = 400;
      m_ge2kap_dialog_sy = 400;
      m_pge2kapDialog = NULL;
      m_bShowge2kap = false;
      

      ::wxDisplaySize(&m_display_width, &m_display_height);

      //    Get a pointer to the opencpn configuration object
      m_pconfig = GetOCPNConfigObject();

      //    And load the configuration items
      LoadConfig();

      // Get a pointer to the opencpn display canvas, to use as a parent for the ge2kap dialog
      m_parent_window = GetOCPNCanvasWindow();

      // m_bge2kapShowIcon = true;
      
      //    This PlugIn needs a toolbar icon, so request its insertion if enabled locally
      if(m_bge2kapShowIcon)
      {
            // wxLogMessage(_T("[GE2KAP] Requesting toolbar icon insertion"));
            m_leftclick_tool_id  = InsertPlugInTool(_T(""), _img_ge2kap, _img_ge2kap, wxITEM_CHECK,
                  _("ge2kap"), _T(""), NULL,
                   GE2KAP_TOOL_POSITION, 0, this);               
      }
                   
      // show the ge2kap dialog
      if(NULL == m_pge2kapDialog)
      {
            wxString plugin_name = _T("GE2KAP_PLUGIN_");
             
            plugin_name.Append(wxString::Format(_T("%d.%d"),GetPlugInVersionMajor(),GetPlugInVersionMinor()));
            // wxLogMessage(_T("[GE2KAP] Init Creating new plugin window"));
            m_pge2kapDialog = new GE2KAPUIDialog();
            m_pge2kapDialog->Create ( m_parent_window, this, -1, plugin_name, m_ge2kap_dir,
                               wxPoint(100,300), wxSize(500,m_ge2kap_dialog_sy));
                               // wxPoint(m_ge2kap_dialog_x,m_ge2kap_dialog_y), wxSize(m_ge2kap_dialog_sx,m_ge2kap_dialog_sy));
            m_pge2kapDialog->Hide();                        // Show modeless, so it stays on the screen
            SetToolbarItemState( m_leftclick_tool_id, true );
            wxLogMessage(_T("[GE2KAP] Init Created new plugin window version:%s"),plugin_name);
      }                   

//      wxMenuItem *pmi = new wxMenuItem(NULL, -1, _("PlugIn Item"));
//      int miid = AddCanvasContextMenuItem(pmi, (PlugInCallBackFunction )&s_ContextMenuCallback );
//      SetCanvasContextMenuItemViz(miid, true);

      return (WANTS_TOOLBAR_CALLBACK    |
           INSTALLS_TOOLBAR_TOOL   |
          // WANTS_CONFIG              |   DEBUG TODO
		     WANTS_PREFERENCES         |           
           WANTS_ONPAINT_VIEWPORT
            );
}

bool ge2kap_pi::DeInit(void)
{
      m_bshuttingDown = true;
                       
      if(m_pge2kapDialog)
      {
         SaveConfig(); 
         m_pge2kapDialog->Close();
         m_pge2kapDialog->Destroy();
      }            

      return true;
}

int ge2kap_pi::GetAPIVersionMajor()
{
      return MY_API_VERSION_MAJOR;
}

int ge2kap_pi::GetAPIVersionMinor()
{
      return MY_API_VERSION_MINOR;
}

int ge2kap_pi::GetPlugInVersionMajor()
{
      return PLUGIN_VERSION_MAJOR;
}

int ge2kap_pi::GetPlugInVersionMinor()
{
      return PLUGIN_VERSION_MINOR;
}

wxBitmap *ge2kap_pi::GetPlugInBitmap()
{
      return _img_ge2kap_pi;
}

wxString ge2kap_pi::GetCommonName()
{
      return _("ge2kap");
}


wxString ge2kap_pi::GetShortDescription()
{
      return _("ge2kap PlugIn for OpenCPN");
}


wxString ge2kap_pi::GetLongDescription()
{
      return _("ge2kap PlugIn for OpenCPN\n\
Communicates with GE2KAP to return the current charts screen coordinates in a GPX file.\n\n\
");

}


void ge2kap_pi::SetDefaults(void)
{
      // If the config somehow says NOT to show the icon, override it so the user gets good feedback
      if(!m_bge2kapShowIcon)
      {
            m_bge2kapShowIcon = true;

            m_leftclick_tool_id  = InsertPlugInTool(_T(""), _img_ge2kap, _img_ge2kap, wxITEM_NORMAL,
                  _("ge2kap"), _T(""), NULL,
                   GE2KAP_TOOL_POSITION, 0, this);
      }
}


int ge2kap_pi::GetToolbarToolCount(void)
{
      return 1;
}


void ge2kap_pi::OnToolbarToolCallback(int id)
{
//      printf("ge2kap_pi ToolCallBack()\n");
//     ::wxBell();

      // Qualify the ge2kap dialog position
            bool b_reset_pos = false;


#ifdef __WXMSW__
        //  Support MultiMonitor setups which an allow negative window positions.
        //  If the requested window does not intersect any installed monitor,
        //  then default to simple primary monitor positioning.
            RECT frame_title_rect;
            frame_title_rect.left =  m_ge2kap_dialog_x;
            frame_title_rect.top =   m_ge2kap_dialog_y;
            frame_title_rect.right = m_ge2kap_dialog_x +m_ge2kap_dialog_sx;
            frame_title_rect.bottom =m_ge2kap_dialog_y + 30;


            if(NULL == MonitorFromRect(&frame_title_rect, MONITOR_DEFAULTTONULL))
                  b_reset_pos = true;
#else
       //    Make sure drag bar (title bar) of window on Client Area of screen, with a little slop...
            wxRect window_title_rect;                    // conservative estimate
            window_title_rect.x =m_ge2kap_dialog_x;
            window_title_rect.y =m_ge2kap_dialog_y;
            window_title_rect.width =m_ge2kap_dialog_sx;
            window_title_rect.height = 30;

            wxRect ClientRect = wxGetClientDisplayRect();
            ClientRect.Deflate(60, 60);      // Prevent the new window from being too close to the edge
            if(!ClientRect.Intersects(window_title_rect))
                  b_reset_pos = true;

#endif

            if(b_reset_pos)
            {
                 m_ge2kap_dialog_x = 20;
                 m_ge2kap_dialog_y = 170;
                 m_ge2kap_dialog_sx = 500;
                 m_ge2kap_dialog_sy = 540;
            }

      //Toggle ge2kap overlay display
      // m_bShowge2kap = !m_bShowge2kap;

      // show the ge2kap dialog
      if(NULL == m_pge2kapDialog)
      {
            // wxLogMessage(_T("[GE2KAP] OnToolbarToolCallback creating new plugin window"));
            m_pge2kapDialog = new GE2KAPUIDialog();
            m_pge2kapDialog->Create ( m_parent_window, this, -1, _("GE2KAP_PLUGIN"),m_ge2kap_dir,
                               wxPoint(100,300), wxSize(500,m_ge2kap_dialog_sy));
                               // wxPoint(m_ge2kap_dialog_x,m_ge2kap_dialog_y), wxSize(m_ge2kap_dialog_sx,m_ge2kap_dialog_sy));
            m_pge2kapDialog->Hide();                        // Show modeless, so it stays on the screen
            SetToolbarItemState( m_leftclick_tool_id, true );
      }

      /*
      //    Toggle dialog?
      if(m_bShowge2kap)
            m_pge2kapDialog->Show();
      else
            m_pge2kapDialog->Hide();
      */      

      m_bShowge2kap = true;  // Make icon active
      // Toggle is handled by the toolbar but we must keep plugin manager b_toggle updated
      // to actual status to ensure correct status upon toolbar rebuild
      SetToolbarItemState( m_leftclick_tool_id, m_bShowge2kap );


}

void ge2kap_pi::Onge2kapDialogClose()
{
      SetToolbarItemState( m_leftclick_tool_id, false );

      if(m_pge2kapDialog)
            m_pge2kapDialog->Hide();

      m_bShowge2kap = false;
}

void ge2kap_pi::SetColorScheme(PI_ColorScheme cs)
{

      DimeWindow(m_pge2kapDialog);

}

void ge2kap_pi::SetCurrentViewPort(PlugIn_ViewPort &vp)
{
      if (NULL == m_pge2kapDialog) return;

      if (vp.clat == m_pastVp.clat && vp.clon == m_pastVp.clon && vp.pix_height == m_pastVp.pix_height && vp.pix_width == m_pastVp.pix_width && vp.rotation == m_pastVp.rotation && vp.chart_scale == m_pastVp.chart_scale && 
            vp.lat_max == m_pastVp.lat_max && vp.lat_min == m_pastVp.lat_min && vp.lon_max == m_pastVp.lon_max && vp.lon_min == m_pastVp.lon_min && vp.view_scale_ppm == m_pastVp.view_scale_ppm)
      {
            return; //Prevents event storm killing the responsiveness. At least in course-up it looks needed.
      }
      // m_pastVp = vp;
      
   
      if (m_bshuttingDown)   
            return; 
                 
       m_pge2kapDialog->my_vp = vp;
       /*
      // LogDebugMessage(wxString::Format(_T("GE2KAP Viewport: Clat=%g, Clon=%g, Width=%d, Height=%d Rotation=%g"), 
      //             vp.clat, vp.clon, vp.pix_width, vp.pix_height, vp.rotation)); 
                  
     // set the viewport coordinates 
     m_pge2kapDialog->m_lat_ul = m_pastVp.lat_max;
     m_pge2kapDialog->m_lon_ul = m_pastVp.lon_min;  // DEBUG CHECK what happens if IDL crossed
     
     m_pge2kapDialog->m_lat_lr = m_pastVp.lat_min;
     m_pge2kapDialog->m_lon_lr = m_pastVp.lon_max;  // DEBUG CHECK what happens if IDL crossed
     
     m_pge2kapDialog->m_lat_ll = m_pastVp.lat_min;
     m_pge2kapDialog->m_lon_ll = m_pastVp.lon_min;  // DEBUG CHECK what happens if IDL crossed
     
     m_pge2kapDialog->m_lat_ur = m_pastVp.lat_max;
     m_pge2kapDialog->m_lon_ur = m_pastVp.lon_max;  // DEBUG CHECK what happens if IDL crossed 
     
     m_pge2kapDialog->skew = m_pastVp.skew;
     m_pge2kapDialog->rotation = m_pastVp.rotation;  
     m_pge2kapDialog->rect = m_pastVp.rv_rect; 
     m_pge2kapDialog->pix_width = m_pastVp.pix_width;  
     m_pge2kapDialog->pix_height = m_pastVp.pix_height;  
     */
          
}

void ge2kap_pi::ShowPreferencesDialog( wxWindow* parent )
{
      ge2kapPrefsDialog *dialog = new ge2kapPrefsDialog( parent, wxID_ANY, _("ge2kap Preferences"), wxPoint( 0,0), wxDefaultSize, wxDEFAULT_DIALOG_STYLE );
      dialog->Fit();
      wxColour cl;
      GetGlobalColor(_T("DILG1"), &cl);
      dialog->SetBackgroundColour(cl);

      dialog->m_cbShowIcon->SetValue(m_bge2kapShowIcon);

      if(dialog->ShowModal() == wxID_OK)
      {
		  // dialog->getValues(); DEBUG ??
        //    Show Icon changed value?
		  if(m_bge2kapShowIcon != dialog->m_cbShowIcon->GetValue())
        {
            m_bge2kapShowIcon= dialog->m_cbShowIcon->GetValue();

            if(m_bge2kapShowIcon) 
            {          
                 // wxLogMessage(_T("[GE2KAP] ShowPreferencesDialog displaying icon"));            
                 m_leftclick_tool_id  = InsertPlugInTool(_T(""), _img_ge2kap, _img_ge2kap,  wxITEM_CHECK,
                        _("ge2kap"), _T(""), NULL,
                         GE2KAP_TOOL_POSITION, 0, this);  
                 SetToolbarItemState( m_leftclick_tool_id, true );         
            }                                   
            else
            {
                  // wxLogMessage(_T("[GE2KAP] ShowPreferencesDialog removing icon"));   
                  RemovePlugInTool(m_leftclick_tool_id);
            } 
        }

            
        SaveConfig();
      }
      delete dialog;
}

bool ge2kap_pi::LoadConfig(void)
{
      wxFileConfig *pConf = (wxFileConfig *)m_pconfig;

      if(pConf)
      {
            pConf->SetPath ( _T( "/Settings/ge2kap" ) );
            pConf->Read ( _T( "Showge2kapIcon" ),  &m_bge2kapShowIcon, 1 );
            return true;
      }
      else
            return false;
}

bool ge2kap_pi::SaveConfig(void)
{
      wxFileConfig *pConf = (wxFileConfig *)m_pconfig;

      if(pConf)
      {
            pConf->SetPath ( _T ( "/Settings/ge2kap" ) );
            pConf->Write ( _T( "Showge2kapIcon" ), m_bge2kapShowIcon );
            return true;
      }
      else
            return false;
}

void LogDebugMessage(wxString msg)
{
      if (DEBUG_MESSAGES)
            wxLogMessage(_T("[GE2KAP] %s"), msg.c_str());
}
