/******************************************************************************
 *
 * Project:  OpenCPN
 * Purpose:  GE2KAP Plugin
 * Author:   Paul Higgins
 *
 ***************************************************************************
 *   Copyright (C) 2010 by Paul Higgins                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA.             *
 ***************************************************************************
 */

#ifndef _GE2KAPPI_H_
#define _GE2KAPPI_H_

#include "wx/wxprec.h"

#ifndef  WX_PRECOMP
  #include "wx/wx.h"
  #include <wx/glcanvas.h>
#endif //precompiled headers

#define     PLUGIN_VERSION_MAJOR    1
#define     PLUGIN_VERSION_MINOR    7

#define     MY_API_VERSION_MAJOR    1
#define     MY_API_VERSION_MINOR    7

#include "ocpn_plugin.h"

#include "ge2kap.h"

#define  DEBUG_MESSAGES       true

void LogDebugMessage(wxString msg);

//----------------------------------------------------------------------------------------------------------
//    The PlugIn Class Definition
//----------------------------------------------------------------------------------------------------------

#define GE2KAP_TOOL_POSITION    -1          // Request default positioning of toolbar tool

class ge2kap_pi : public opencpn_plugin_17
{
public:
      ge2kap_pi(void *ppimgr);
      ~ge2kap_pi(void);

//    The required PlugIn Methods
      int Init(void);
      bool DeInit(void);

      int GetAPIVersionMajor();
      int GetAPIVersionMinor();
      int GetPlugInVersionMajor();
      int GetPlugInVersionMinor();
      wxBitmap *GetPlugInBitmap();
      wxString GetCommonName();
      wxString GetShortDescription();
      wxString GetLongDescription();


      void SetDefaults(void);

      int GetToolbarToolCount(void);

      void OnToolbarToolCallback(int id);


// Other public methods

      void Onge2kapDialogClose();
      void SetColorScheme(PI_ColorScheme cs);
      void Setge2kapDir(wxString ge2kap_dir){ m_ge2kap_dir = ge2kap_dir;};
      void Setge2kapDialogX    (int x){ m_ge2kap_dialog_x = x;};
      void Setge2kapDialogY    (int x){ m_ge2kap_dialog_y = x;}
      void Setge2kapDialogSizeX(int x){ m_ge2kap_dialog_sx = x;}
      void Setge2kapDialogSizeY(int x){ m_ge2kap_dialog_sy = x;}     
      void SetCurrentViewPort(PlugIn_ViewPort &vp); 
      void ShowPreferencesDialog( wxWindow* parent );

private:
      bool LoadConfig(void);
      bool SaveConfig(void);
      

      wxFileConfig     *m_pconfig;
      wxWindow         *m_parent_window;

      GE2KAPUIDialog     *m_pge2kapDialog;

      int              m_display_width, m_display_height;
      int              m_leftclick_tool_id;

      bool             m_bShowge2kap;

      int              m_ge2kap_dialog_x, m_ge2kap_dialog_y;
      int              m_ge2kap_dialog_sx, m_ge2kap_dialog_sy;
      wxString         m_ge2kap_dir;
      bool              m_bge2kapShowIcon;
      PlugIn_ViewPort   m_pastVp;
      bool              m_bshuttingDown;

};

#endif



